﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using InteractiveData.eSignalDesktop.Trading;

namespace CQGPlugin.SymbolSearch
{
    internal static class CqgSymbolFilteringExtensions
    {
        public static CqgSymbol TakeWithMaximumRelevance(this IEnumerable<CqgSymbol> symbols, IESignalSymbol eSignalSymbol)
        {
            var symbolsList = symbols.ToList();

            if (eSignalSymbol.Type == SymbolType.Spread)
            {
                var distanceInMonths = Math.Abs(DistanceBetweenLegs(eSignalSymbol));

                symbolsList = FilterByDistanceInMonths(distanceInMonths, symbolsList);
            }

            return symbolsList
                .OrderByDescending(responseSymbol => DescriptionRelevance(eSignalSymbol, responseSymbol.Description))
                .ThenByDescending(responseSymbol => RootRelevance(eSignalSymbol.Root, responseSymbol.Root))
                .First();
        }

        private static List<CqgSymbol> FilterByDistanceInMonths(int distanceInMonths, IEnumerable<CqgSymbol> symbolsList)
        {
            var rootEnd = distanceInMonths.ToString(CultureInfo.InvariantCulture) + "M";
            return symbolsList.Where(symbol => symbol.Root.EndsWith(rootEnd)).ToList();
        }

        private static int DistanceBetweenLegs(IESignalSymbol eSignalSymbol)
        {
            var legs = eSignalSymbol.Legs.ToArray();

            return ((legs[0].ExpirationYear - legs[1].ExpirationYear) * 12)
                + legs[0].ExpirationMonth - legs[1].ExpirationMonth;
        }

        private static double RootRelevance(string eSignalRoot, string cqgRoot)
        {
            if (cqgRoot.StartsWith(eSignalRoot))
            {
                return (double) eSignalRoot.Length / cqgRoot.Length;
            }

            return 0.0;
        }

        private static double DescriptionRelevance(IESignalSymbol eSignalSymbol, string cqgDescription)
        {
            var keyWords = eSignalSymbol.Description.SplitToWords().WithoutCommonWords(eSignalSymbol).ToList();

            var responseWords = cqgDescription.SplitToWords().ToArray();

            var countOfQueryWordsInResponse = responseWords.Intersect(keyWords).Count();

            return (double)countOfQueryWordsInResponse / responseWords.Count();
        }
    }
}