﻿using System;
using System.Collections.Generic;
using System.Linq;
using InteractiveData.eSignalDesktop.Trading;

namespace CQGPlugin.SymbolSearch
{
    internal static class QueryWordsUtils
    {
        public static IEnumerable<string> MakeWordsForQuery(IESignalSymbol symbol)
        {
            return symbol.Description
                .SplitToWords()
                .WithAdditionalWords(symbol)
                .WithoutCommonWords(symbol)
                .WithRestrictionWords(symbol);
        }

        public static IEnumerable<string> SplitToWords(this string value)
        {
            return value.ToUpper().Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries)
                    .Where(NotPunctuation);
        }

        public static IEnumerable<string> WithoutCommonWords(this IEnumerable<string> queryWords, IESignalSymbol symbol)
        {
            return queryWords
                .Except(EsignalDescriptionCommonWordsForSymbolType(symbol.Type))
                .Where(word => word != symbol.ExchangeName);
        }

        private static IEnumerable<string> WithAdditionalWords(this IEnumerable<string> sourceQuery, IESignalSymbol symbol)
        {
            var sourceQueryList = sourceQuery.ToList();
         
            if (symbol.Type == SymbolType.Spread && !sourceQueryList.Contains("SPREAD"))
            {
                sourceQueryList.Add("SPREAD");
            }

            return sourceQueryList;
        }

        private static IEnumerable<string> WithRestrictionWords(this IEnumerable<string> sourceQuery, IESignalSymbol symbol)
        {
            var restrictionWords = new []
            {
                "BLOCKS",
                "SETTLEMENT",
                "COMBINED",
                "MARKER",
                "BUTTERFLY",
                "FINANCIAL",
                "STRIP",
                "VARIANCE",
            };

            var sourceQueryList = sourceQuery.ToList();

            if (symbol.Type != SymbolType.Spread)
            {
                sourceQueryList.Add("NOT SPREAD");
                sourceQueryList.Add("NOT CALENDAR");
            }

            return sourceQueryList.Concat(restrictionWords.Except(sourceQueryList).Select(MakeNegation));
        }

        private static string[] EsignalDescriptionCommonWordsForSymbolType(SymbolType symbolType)
        {
            if (symbolType == SymbolType.Future)
            {
                return new []{"FUTURES"};
            }

            if (symbolType == SymbolType.FutureOption)
            {
                return new []{"OPTIONS"};
            }

            if (symbolType == SymbolType.Spread)
            {
                return new []{"FUTURE", "FUTURES"};
            }

            throw new ArgumentOutOfRangeException();
        }

        private static bool NotPunctuation(string word)
        {
            return word != "-";
        }

        private static string MakeNegation(string word)
        {
            return "NOT " + word;
        }
    }
}
