﻿using System;
using System.Collections.Specialized;
using System.Globalization;
using System.Linq;
using System.Web;
using InteractiveData.eSignalDesktop.Trading;

namespace CQGPlugin.SymbolSearch
{
    internal static class SearchQueryBuilder
    {
        public static NameValueCollection PrepareQuery(IESignalSymbol symbol)
        {
            var query = string.Join(" ", QueryWordsUtils.MakeWordsForQuery(symbol));

            var result = PrepareQuery(query);

            result.Add(ExchangeFilter(symbol));
            result.Add(SymbolTypeSepecificFilters(symbol));

            return result;
        }
        
        public static NameValueCollection PrepareQuery(string searchString)
        {
            var result = new NameValueCollection
            {
                { searchKey, HttpUtility.UrlEncode(searchString) },
                { rowsKey, "10000" }
            };

            return result;
        }

        private static NameValueCollection ExchangeFilter(IESignalSymbol symbol)
        {
            var exchange = ExchangesMap.MapEsingalExchangeToCQGExchange(symbol.ExchangeName);

            if (exchange != null)
            {
                return new NameValueCollection {{ exchangeFilterKey, exchange }};
            }

            return new NameValueCollection();
        }

        private static NameValueCollection SymbolTypeSepecificFilters(IESignalSymbol symbol)
        {
            if (symbol.Type == SymbolType.Future)
            {
                return FuturesSpecificFilters(symbol);
            }

            if (symbol.Type == SymbolType.FutureOption)
            {
                return FutureOptionSpecificFilters(symbol);
            }

            if (symbol.Type == SymbolType.Spread)
            {
                return FutureSpreadSpecificFilters(symbol);
            }

            throw new ArgumentOutOfRangeException("symbol");
        }

        private static NameValueCollection FutureSpreadSpecificFilters(IESignalSymbol symbol)
        {
            var result = new NameValueCollection
            {
                { typeFilterKey, "Spread" },

                FiltersForSymbolWithExpiration(symbol.Legs.First().ExpirationMonth, symbol.Legs.First().ExpirationYear)
            };

            return result;
        }

        private static NameValueCollection FutureOptionSpecificFilters(IESignalSymbol symbol)
        {
            var result = new NameValueCollection
            {
                { typeFilterKey, "Future Option" },
                { optionTypeFilterKey, symbol.OptionIsCall ? "Call" : "Put" },
                { strikeFilterKey, symbol.OptionStrike.ToString("F6", CultureInfo.InvariantCulture) },

                FiltersForSymbolWithExpiration(symbol.ExpirationMonth, symbol.ExpirationYear)
            };
            
            return result;
        }

        private static NameValueCollection FuturesSpecificFilters(IESignalSymbol symbol)
        {
            var result = new NameValueCollection
            {
                { typeFilterKey, "Future" },

                FiltersForSymbolWithExpiration(symbol.ExpirationMonth, symbol.ExpirationYear)
            };

            return result;
        }

        private static NameValueCollection FiltersForSymbolWithExpiration(int expirationMonth, int expirationYear)
        {
            var result = new NameValueCollection
            {
                { monthFilterKey, ((Months) expirationMonth).ToString() },
                { yearFilterKey, expirationYear.ToString(CultureInfo.InvariantCulture) }
            };

            return result;
        }

        private const string searchKey = "s";
        private const string rowsKey = "rows";
        private const string monthFilterKey = "filter.month";
        private const string yearFilterKey = "filter.year";
        private const string typeFilterKey = "filter.type";
        private const string strikeFilterKey = "filter.strike";
        private const string optionTypeFilterKey = "filter.optiontype";
        private const string exchangeFilterKey = "filter.exchange";
    }
}