﻿using System;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using InteractiveData.eSignalDesktop.Trading;

namespace CQGPlugin.SymbolSearch
{
    internal static class SymbolSearch
    {
        [DataContract]
        private struct Response
        {
            [DataMember(Name = "numFound")]
            public int ResultsCount { get; set; }
            
            [DataMember(Name = "offset")]
            public int Offset { get; set; }

            [DataMember(Name = "size")]
            public int Size { get; set; }

            [DataMember(Name = "query")]
            public string Query { get; set; }

            [DataMember(Name = "results")]
            public CqgSymbol[] Results { get; set; }
        }

        public static CqgSymbol Search(IESignalSymbol symbol)
        {
            var response = Query(SearchQueryBuilder.PrepareQuery(symbol));

            return response.Results.TakeWithMaximumRelevance(symbol);
        }

        public static CqgSymbol SearchByCqgSymbol(string symbol)
        {
            var response = Query(SearchQueryBuilder.PrepareQuery(symbol));

            return response.Results.First();
        }

        private static Response Query(NameValueCollection query)
        {
            using (var webClient = new WebClient { QueryString = query })
            {
                var stream = webClient.OpenRead(baseUrl);

                if (stream == null)
                {
                    throw new InvalidOperationException("Can't request server");
                }

                return (Response)serializer.ReadObject(stream);    
            }
        }

        private static readonly DataContractJsonSerializer serializer = new DataContractJsonSerializer(
            typeof (Response), new[] {typeof (CqgSymbol)});

        private const string baseUrl = "http://lookup.qa2.marketcenter.com/cqg/search";
    }
}
